///////////////////////////////////////////////////////////////////////////////
//
//      ##  ######
//       ######  ###
//  ## ###############        Shark 3D Engine (www.shark3d.com)
//   ########## # # #
//    ########                Copyright (c) 1996-2006 Spinor GmbH.
//   ######### # # #          All rights reserved.
//  ##   ##########
//      ##
//
///////////////////////////////////////////////////////////////////////////////

//@cpp

#ifndef GTP_SHADER_WORLDMAT_H
#define GTP_SHADER_WORLDMAT_H

#include "../../interf/eng_shader.h"
#include "../../interf/eng_shaderenv.h"
#include "../../interf/eng_pos.h"
#include "../../../comp/core/comp_setup.h"
#include "../../../comp/util/comp_util_uniquebase.h"

///////////////////////////////////////////////////////////////////////////////

class s3d_CEngShaderSpecialWorldMatGfx;

//@
typedef s3d_CUtilKeyEntry<s3d_CEngGfxCtxPtr, s3d_CEngShaderSpecialWorldMatGfx*>
        s3d_CEngShaderSpecialWorldMatGfxEntry;

//@
typedef s3d_CUtilTree<s3d_CEngShaderSpecialWorldMatGfxEntry>
        s3d_CEngShaderSpecialWorldMatGfxTree;

///////////////////////////////////////////////////////////////////////////////

struct s3d_CEngShaderSpecialWorldMatData: s3d_CUtilPtrBase
{
    s3d_CUtilStr m_Info;
    s3d_CUtilAtomArray m_MethodArray;
    s3d_CUtilStr m_WorldMatVarInfo;
    s3d_CUtilAtomPtr m_WorldMatVarAtom;
	bool m_Inverse;
	bool m_Transpose;
};

typedef s3d_CUtilPtr<s3d_CEngShaderSpecialWorldMatData>
        s3d_CEngShaderSpecialWorldMatDataPtr;

///////////////////////////////////////////////////////////////////////////////

//@
class s3d_CEngShaderSpecialWorldMat
    : public s3d_CEngShader,
    public s3d_CCompUtilUniqueBase,
    public s3d_CCompSetup
{
public:
    S3D_UTIL_RTTI_TABLE_DECLARE

    //@
    s3d_CEngShaderSpecialWorldMat();

    //@
    void SetupInit(
            s3d_CUtilMsgHandler *MsgHandler, s3d_CUtilStr_cr Info,
            s3d_CCompSuppl *FetchSuppl, s3d_CUtilSnkChunk *Param);

    //@
    void SetupDone();

    //@
    s3d_CUtilStr GetInfo();

    //@
    void RegisterNotif(s3d_CUtilNotifRecip *Recip);

    //@
    s3d_CEngShaderGfxPtr FindShaderGfx(
            s3d_CUtilNotifGather *NotifGather, s3d_CEngGfxCtx *GfxCtx);

private:
    s3d_CEngShaderEnvPtr m_ShaderEnv;
    s3d_CEngShaderSpecialWorldMatGfxTree m_ShaderGfxTree;
    s3d_CEngShaderSpecialWorldMatDataPtr m_Data;
};

//@
typedef s3d_CUtilPtr<s3d_CEngShaderSpecialWorldMat>
        s3d_CEngShaderSpecialWorldMatPtr;

///////////////////////////////////////////////////////////////////////////////

//@
class s3d_CEngShaderSpecialWorldMatGfx: public s3d_CEngShaderGfx
{
public:
    //@
    s3d_CEngShaderSpecialWorldMatGfx(
            s3d_CEngShaderEnv *ShaderEnv, s3d_CEngGfxCtx *GfxCtx,
            s3d_CEngShaderSpecialWorldMatData *Data);

    //@
    s3d_CEngShaderSpecialWorldMatGfxTree::CNode *GetNode();

    //@
    void RegisterNotif(s3d_CUtilNotifRecip *Recip);

    //@
    void ExtractGfx();

    //@
    bool IsValid();

    //@
    void LinkCollect(s3d_CUtilAtomSet &VarAtomSet);

    //@
    void LinkAppoint(s3d_CDrvVarDecl *VarDecl);

    //@
    s3d_CEngShaderInstPtr CreateShaderInst(
            s3d_CEngShaderHead *Head);

private:
    s3d_CEngShaderSpecialWorldMatGfxTree::CNode m_Node;
    s3d_CEngShaderEnvPtr m_ShaderEnv;
    s3d_CEngShaderSpecialWorldMatDataPtr m_Data;

    // ShaderGfx-Data Variables:
    s3d_CDrvVarSlotInit m_WorldMatVarSlot;

    friend class s3d_CEngShaderSpecialWorldMatInst;
};

//@
typedef s3d_CUtilPtr<s3d_CEngShaderSpecialWorldMatGfx>
        s3d_CEngShaderSpecialWorldMatGfxPtr;

///////////////////////////////////////////////////////////////////////////////

//@
class s3d_CEngShaderSpecialWorldMatInst
    : public s3d_CEngShaderInst,
    public s3d_CEngShaderExec
{
public:
    //@
    s3d_CEngShaderSpecialWorldMatInst(
            s3d_CEngShaderEnv *ShaderEnv, 
            s3d_CEngShaderSpecialWorldMatGfx *ShaderGfx);

    //@
    void RegisterNotif(s3d_CUtilNotifRecip *Recip);

    //@
    void SetShaderCtx(s3d_CEngShaderCtx *ShaderCtx);

    //@
    void SetGfxState(s3d_CEngGfxState *GfxState);

    //@
    void SetPos(s3d_CEngPos *Pos);

    //@
    void SetModel(s3d_CEngModel *Model);

    //@
    void ArrangeCollect(
            s3d_CUtilMemPool *MemPool,
            s3d_CEngShaderScopeArray &ScopeArray);

    //@
    void ArrangeInit(
            s3d_CUtilMemPool *MemPool,
            s3d_CEngShaderScopeArray_cr ScopeArray);

    //@
    void ArrangePerform(
            s3d_CUtilMemPool *MemPool,
            s3d_CEngShaderScopeArray_cr ScopeArray);

    //@
    void ArrangeFeedback(
            s3d_CUtilMemPool *MemPool,
            s3d_CEngShaderScopeArray_cr ScopeArray);

    //@
    void ArrangeGather(
            s3d_CEngShaderExecArray &ExecArray);

    //@
    void ShaderExecPerform(
            s3d_CEngShaderScope *Scope, 
            s3d_CUtilAtom *Method,
            s3d_CEngGfxTaskArray &TaskArray);

private:
    s3d_CEngShaderEnvPtr m_ShaderEnv;
    s3d_CEngShaderSpecialWorldMatGfxPtr m_ShaderGfx;
    s3d_CEngPosPtr m_Pos;

    void SetVars(
            s3d_CUtilMemPool *MemPool, s3d_CEngShaderScope *Scope);
};

//@
typedef s3d_CUtilPtr<s3d_CEngShaderSpecialWorldMatInst>
        s3d_CEngShaderSpecialWorldMatInstPtr;

///////////////////////////////////////////////////////////////////////////////

#endif
