///////////////////////////////////////////////////////////////////////////////
//
//      ##  ######
//       ######  ###
//  ## ###############        Shark 3D Engine (www.shark3d.com)
//   ########## # # #
//    ########                Copyright (c) 1996-2006 Spinor GmbH.
//   ######### # # #          All rights reserved.
//  ##   ##########
//      ##
//
///////////////////////////////////////////////////////////////////////////////

//@cpp

#ifndef GTP_SHADER_FOCUSEPROJMAT_H
#define GTP_SHADER_FOCUSEDPROJMAT_H

#include "../../interf/eng_shader.h"
#include "../../interf/eng_shaderenv.h"
#include "../../interf/eng_pos.h"
#include "../../../comp/core/comp_setup.h"
#include "../../../comp/util/comp_util_uniquebase.h"

///////////////////////////////////////////////////////////////////////////////

class gtp_Shader_FocusedProjMatGfx;

//@
typedef s3d_CUtilKeyEntry<s3d_CEngGfxCtxPtr, gtp_Shader_FocusedProjMatGfx*>
        gtp_Shader_FocusedProjMatGfxEntry;

//@
typedef s3d_CUtilTree<gtp_Shader_FocusedProjMatGfxEntry>
        gtp_Shader_FocusedProjMatGfxTree;

///////////////////////////////////////////////////////////////////////////////

struct gtp_Shader_FocusedProjMatData: s3d_CUtilPtrBase
{
    s3d_CUtilStr m_Info;
    s3d_CUtilAtomArray m_MethodArray;
    s3d_CUtilStr m_FocusedProjMatVarInfo;
    s3d_CUtilAtomPtr m_FocusedProjMatVarAtom;
	s3d_CUtilStr m_CenterPointVarInfo;
    s3d_CUtilAtomPtr m_CenterPointVarAtom;	
	s3d_CUtilStr m_MeshVarInfo;
    s3d_CUtilAtomPtr m_MeshVarAtom;
   
};

typedef s3d_CUtilPtr<gtp_Shader_FocusedProjMatData>
        gtp_Shader_FocusedProjMatDataPtr;

///////////////////////////////////////////////////////////////////////////////

//@
class gtp_Shader_FocusedProjMat
    : public s3d_CEngShader,
    public s3d_CCompUtilUniqueBase,
    public s3d_CCompSetup
{
public:
    S3D_UTIL_RTTI_TABLE_DECLARE

    //@
    gtp_Shader_FocusedProjMat();

    //@
    void SetupInit(
            s3d_CUtilMsgHandler *MsgHandler, s3d_CUtilStr_cr Info,
            s3d_CCompSuppl *FetchSuppl, s3d_CUtilSnkChunk *Param);

    //@
    void SetupDone();

    //@
    s3d_CUtilStr GetInfo();

    //@
    void RegisterNotif(s3d_CUtilNotifRecip *Recip);

    //@
    s3d_CEngShaderGfxPtr FindShaderGfx(
            s3d_CUtilNotifGather *NotifGather, s3d_CEngGfxCtx *GfxCtx);

private:
    s3d_CEngShaderEnvPtr m_ShaderEnv;
    gtp_Shader_FocusedProjMatGfxTree m_ShaderGfxTree;
    gtp_Shader_FocusedProjMatDataPtr m_Data;
};

//@
typedef s3d_CUtilPtr<gtp_Shader_FocusedProjMat>
        gtp_Shader_FocusedProjMatPtr;

///////////////////////////////////////////////////////////////////////////////

//@
class gtp_Shader_FocusedProjMatGfx: public s3d_CEngShaderGfx
{
public:
    //@
    gtp_Shader_FocusedProjMatGfx(
            s3d_CEngShaderEnv *ShaderEnv, s3d_CEngGfxCtx *GfxCtx,
            gtp_Shader_FocusedProjMatData *Data);

    //@
    gtp_Shader_FocusedProjMatGfxTree::CNode *GetNode();

    //@
    void RegisterNotif(s3d_CUtilNotifRecip *Recip);

    //@
    void ExtractGfx();

    //@
    bool IsValid();

    //@
    void LinkCollect(s3d_CUtilAtomSet &VarAtomSet);

    //@
    void LinkAppoint(s3d_CDrvVarDecl *VarDecl);

    //@
    s3d_CEngShaderInstPtr CreateShaderInst(
            s3d_CEngShaderHead *Head);

private:
    gtp_Shader_FocusedProjMatGfxTree::CNode m_Node;
    s3d_CEngShaderEnvPtr m_ShaderEnv;
    gtp_Shader_FocusedProjMatDataPtr m_Data;

    // ShaderGfx-Data Variables:
    s3d_CDrvVarSlotInit m_FocusedProjMatVarSlot;
	s3d_CDrvVarSlotInit m_CenterPointVarSlot;
	s3d_CDrvVarSlotInit m_MeshVarSlot;
   
    friend class gtp_Shader_FocusedProjMatInst;
};

//@
typedef s3d_CUtilPtr<gtp_Shader_FocusedProjMatGfx>
        gtp_Shader_FocusedProjMatGfxPtr;

///////////////////////////////////////////////////////////////////////////////

//@
class gtp_Shader_FocusedProjMatInst
    : public s3d_CEngShaderInst,
    public s3d_CEngShaderExec
{
public:
    //@
    gtp_Shader_FocusedProjMatInst(
            s3d_CEngShaderEnv *ShaderEnv, 
            gtp_Shader_FocusedProjMatGfx *ShaderGfx);

    //@
    void RegisterNotif(s3d_CUtilNotifRecip *Recip);

    //@
    void SetShaderCtx(s3d_CEngShaderCtx *ShaderCtx);

    //@
    void SetGfxState(s3d_CEngGfxState *GfxState);

    //@
    void SetPos(s3d_CEngPos *Pos);

    //@
    void SetModel(s3d_CEngModel *Model);

    //@
    void ArrangeCollect(
            s3d_CUtilMemPool *MemPool,
            s3d_CEngShaderScopeArray &ScopeArray);

    //@
    void ArrangeInit(
            s3d_CUtilMemPool *MemPool,
            s3d_CEngShaderScopeArray_cr ScopeArray);

    //@
    void ArrangePerform(
            s3d_CUtilMemPool *MemPool,
            s3d_CEngShaderScopeArray_cr ScopeArray);

    //@
    void ArrangeFeedback(
            s3d_CUtilMemPool *MemPool,
            s3d_CEngShaderScopeArray_cr ScopeArray);

    //@
    void ArrangeGather(
            s3d_CEngShaderExecArray &ExecArray);

    //@
    void ShaderExecPerform(
            s3d_CEngShaderScope *Scope, 
            s3d_CUtilAtom *Method,
            s3d_CEngGfxTaskArray &TaskArray);

private:
    s3d_CEngShaderEnvPtr m_ShaderEnv;
    gtp_Shader_FocusedProjMatGfxPtr m_ShaderGfx;
    s3d_CEngPosPtr m_Pos;

    void SetVars(
            s3d_CUtilMemPool *MemPool, s3d_CEngShaderScope *Scope);
};

//@
typedef s3d_CUtilPtr<gtp_Shader_FocusedProjMatInst>
        gtp_Shader_FocusedProjMatInstPtr;

///////////////////////////////////////////////////////////////////////////////

#endif
